#version 330

in vec2 TexCoord0;

out vec4 FragColor;

uniform sampler2D Texture0;

void main()
{
	/*
	 * Adapted from:
	 * http://code.google.com/p/processing/source/browse/trunk/processing/java/libraries/opengl/examples/Shaders/FXAA/data/fxaa.glsl?spec=svn9724&r=9668
	 */

	
	float FXAA_SPAN_MAX = 8.0;
	float FXAA_REDUCE_MUL = 1.0/8.0;
	//float FXAA_REDUCE_MUL = 0;
	float FXAA_REDUCE_MIN = 1.0/128.0;
	//vec2 texcoordOffset = vec2(1.0/1024.0, 1.0/768); 
	vec2 texcoordOffset = 1.0 / textureSize(Texture0, 0);

	vec3 rgbNW = textureOffset(Texture0, TexCoord0.xy, ivec2(-1, -1)).xyz;
	vec3 rgbNE = textureOffset(Texture0, TexCoord0.xy, ivec2(+1, -1)).xyz;
	vec3 rgbSW = textureOffset(Texture0, TexCoord0.xy, ivec2(-1, +1)).xyz;
	vec3 rgbSE = textureOffset(Texture0, TexCoord0.xy, ivec2(+1, +1)).xyz;
	vec3 rgbM = texture2D(Texture0, TexCoord0.xy).xyz;

	vec3 luma = vec3(0.299, 0.587, 0.114);
	float lumaNW = dot(rgbNW, luma);
	float lumaNE = dot(rgbNE, luma);
	float lumaSW = dot(rgbSW, luma);
	float lumaSE = dot(rgbSE, luma);
	float lumaM  = dot(rgbM , luma);

	float lumaMin = min(lumaM, min(min(lumaNW, lumaNE), min(lumaSW, lumaSE)));
	float lumaMax = max(lumaM, max(max(lumaNW, lumaNE), max(lumaSW, lumaSE)));

	vec2 dir;
	dir.x = -((lumaNW + lumaNE) - (lumaSW + lumaSE));
	dir.y =  ((lumaNW + lumaSW) - (lumaNE + lumaSE));

	float dirReduce = max((lumaNW + lumaNE + lumaSW + lumaSE) * (0.25 * FXAA_REDUCE_MUL), FXAA_REDUCE_MIN);

	float rcpDirMin = 1.0/(min(abs(dir.x), abs(dir.y)) + dirReduce);

	dir =	min(vec2(FXAA_SPAN_MAX, FXAA_SPAN_MAX),
			max(vec2(-FXAA_SPAN_MAX, -FXAA_SPAN_MAX), dir * rcpDirMin)) * texcoordOffset;

	vec3 rgbA = (1.0/2.0) * (
		texture2D(Texture0, TexCoord0.xy + dir * (1.0/3.0 - 0.5)).xyz +
		texture2D(Texture0, TexCoord0.xy + dir * (2.0/3.0 - 0.5)).xyz);
	vec3 rgbB = rgbA * (1.0/2.0) + (1.0/4.0) * (
		texture2D(Texture0, TexCoord0.xy + dir * (0.0/3.0 - 0.5)).xyz +
		texture2D(Texture0, TexCoord0.xy + dir * (3.0/3.0 - 0.5)).xyz);
	float lumaB = dot(rgbB, luma);

	vec4 result;
	if((lumaB < lumaMin) || (lumaB > lumaMax)){
		result.xyz=rgbA;
	} else {
		result.xyz=rgbB;
	}
	result.a = 1.0;

	FragColor = result;
}
