#version 330

in vec2 TexCoord0;
in vec3 Normal0;
in vec3 Tangent0;
in vec3 WorldPos0;

layout (location = 0) out vec4 ColorOut;
layout (location = 1) out vec3 NormalOut;
layout (location = 2) out vec3 NormalPureOut;
layout (location = 3) out float MaterialOut;

uniform sampler2D DiffuseTexture; // diffuse texture
uniform sampler2D SpecularTexture; // specular map
uniform sampler2D NormalTexture; // normal map

uniform float SpecularPower;
uniform bool UseNormalMapping;

void main()
{
	vec3 diffuseColor = texture(DiffuseTexture, TexCoord0).xyz;
	float specIntensity = texture(SpecularTexture, TexCoord0).r;

	ColorOut = vec4(diffuseColor.xyz, specIntensity);
	NormalPureOut = normalize(Normal0);

	// Normal mapping:
	vec3 normalTS = texture(NormalTexture, TexCoord0).xyz;

	if (UseNormalMapping)
	{
		normalTS = 2.0f * normalTS - 1.0f; // convert from (0, 1) to (-1, +1)

		vec3 normalWorld = normalize(Normal0);
		vec3 tangentWorld = normalize(Tangent0);
		vec3 bitangentWorld = normalize(cross(normalWorld, tangentWorld));

		// convert from tangent space to world space:
		mat3 TBN = mat3(tangentWorld, bitangentWorld, normalWorld);
		NormalOut = normalize(TBN * normalTS);
	}
	else
	{
		NormalOut = normalize(Normal0);
	}

	MaterialOut = SpecularPower / 65536.0f;
}

