#version 330 core

layout(location = 0) in vec3 Position;

uniform mat4 ModelViewProjection;
uniform mat4 model;

uniform float time;
uniform vec3 cameraPos;

out vec4 Wave0;
out vec2 Wave1;
out vec2 Wave2;
out vec2 Wave3;

out vec4 ScreenPos;
out vec3 Eye;
out vec3 FragPos;
out vec3 Normal;

const mat3 worldToTangentSpace = mat3(vec3(0, 1, 0), vec3(0, 0, 1), vec3(1, 0, 0));

void main()
{
	gl_Position = ModelViewProjection * vec4(Position, 1.0f);
	FragPos = vec3(model * vec4(Position, 1.0f));

	vec2 translation = vec2(0.0, mod(time, 3000)*0.0007);
	vec2 vTexCoords = FragPos.xz*0.001;

	// Scale texture coordinates to get mix of low/high frequency details
	Wave0.xy = vTexCoords.xy + translation*20.0;
	Wave1 = vTexCoords.xy*20.0 + translation*40.0;;
	Wave2 = vTexCoords.xy*40.0 + translation*20.0;
	Wave3 = vTexCoords.xy*80.0 + translation;
	
	// perspective corrected projection      
	vec4 vHPos = gl_Position;
	Wave0.w = vHPos.w;
	vHPos.y = -vHPos.y;
	ScreenPos.xy = (vHPos.xy + vHPos.w)*0.5;
	ScreenPos.zw = vec2(1.0, vHPos.w);


	vec3 EyeVec = cameraPos.xyz - FragPos;
	
	Eye = worldToTangentSpace * EyeVec;
}